
// storage.c :  Plug-in for playing from intel "S33" serial flash eeprom. 
// For this example, a QH25F640S33B8 chip is connected to SI, SO, SCLK, XCS.

// This version overrides the main player loop as an example on how to take
// full control of the playing

#include <stdlib.h>
#include <vs1000.h>
#include <minifat.h>
#include <player.h>


#define SPI_EEPROM_COMMAND_READ_STATUS_REGISTER  0x05
#define SPI_EEPROM_COMMAND_READ  0x03

//Set SPI to MASTER; 8BIT; FSYNC Idle => xCS high
#define SPI_MASTER_8BIT_CSHI   PERIP(SPI0_CONFIG) = \
        SPI_CF_MASTER | SPI_CF_DLEN8 | SPI_CF_FSIDLE1

//Set SPI to MASTER; 8BIT; FSYNC not Idle => xCS low
#define SPI_MASTER_8BIT_CSLO   PERIP(SPI0_CONFIG) = \
        SPI_CF_MASTER | SPI_CF_DLEN8 | SPI_CF_FSIDLE0

//Set SPI to MASTER; 16BIT; FSYNC not Idle => xCS low
#define SPI_MASTER_16BIT_CSLO  PERIP(SPI0_CONFIG) = \
        SPI_CF_MASTER | SPI_CF_DLEN16 | SPI_CF_FSIDLE0


void InitSpi() {
  SPI_MASTER_8BIT_CSHI;
  PERIP(SPI0_FSYNC) = 0;        // Frame Sync is used as an active low xCS
  PERIP(SPI0_CLKCONFIG) = SPI_CC_CLKDIV * (1-1);   // Spi clock divider = 1
  PERIP(GPIO1_MODE) |= 0x1f;    // Set SPI pins to be peripheral controlled
}  


void EESingleCycleCommand(u_int16 cmd){
  SPI_MASTER_8BIT_CSHI; 
  SPI_MASTER_8BIT_CSLO;
  SpiSendReceive(cmd);
  SPI_MASTER_8BIT_CSHI;
}


/// Wait for not_busy (status[0] = 0) and return status
u_int16 EEWaitGetStatus(void) {
  u_int16 status;
  SPI_MASTER_8BIT_CSHI;
  SPI_MASTER_8BIT_CSLO;
  SpiSendReceive(SPI_EEPROM_COMMAND_READ_STATUS_REGISTER);
  while ((status = SpiSendReceive(0)) & 0x01)
    ; //Wait until ready
  SPI_MASTER_8BIT_CSHI;
  return status;
}


/// Read a block from EEPROM
/// \param blockn number of 512-byte sector 0..32767
/// \param dptr pointer to data block
u_int16 EEReadBlock(u_int16 blockn, u_int16 *dptr) {
  EEWaitGetStatus();                    // Wait until EEPROM is not busy
  SPI_MASTER_8BIT_CSLO;                  // Bring xCS low
  SpiSendReceive(SPI_EEPROM_COMMAND_READ);
  SpiSendReceive(blockn>>7);            // Address[23:16] = blockn[14:7]
  SpiSendReceive((blockn<<1)&0xff);     // Address[15:8]  = blockn[6:0]0
  SpiSendReceive(0);                    // Address[7:0]   = 00000000
  SPI_MASTER_16BIT_CSLO;                 // Switch to 16-bit mode
  { int n;
    for (n=0; n<256; n++){
      *dptr++ = SpiSendReceive(0);      // Receive Data
    }
  }
  SPI_MASTER_8BIT_CSHI;                  // Bring xCS back to high
  return 0;
}


// Disk image is prommed to EEPROM at sector 0x80 onwards. 
#define FAT_START_SECTOR 0x80

// This function will replace ReadDiskSector() functionality
auto u_int16 MyReadDiskSector(register __i0 u_int16 *buffer,
			      register __a u_int32 sector) {
  EEReadBlock(sector+FAT_START_SECTOR, buffer);
  return 0;
}

// Initialize SPI and hook in own disk read Function.
// This example plays ogg files from a FAT image that has been
// previously written to a serial EEPROM.

void main(void) {
  InitSpi();
  // Hook in own disk sector read function
  SetHookFunction((u_int16)ReadDiskSector, MyReadDiskSector);

  if (InitFileSystem()) {
    // File System Not Found, play low sine wave as an "error message"
    register u_int16 *g = (u_int16*) g_yprev0; // get ptr to sintest params
    *g++ = 4; *g++ = 44100; *g++ = 0x5050; *g++ = 120; *g++ = 200;
    SinTest(); //Exit to SinTest: Play weird tone at low volume

  } else {

    // FAT File System Found, play files from it.
    // The code below is an example playing routine. It's provided
    // here as an example of one possible way of implementing the player
    // main routine. Its function is mainly to select the number of file
    // to be played, open it via OpenFile() and play it via PlayCurrentFile().

    // The code below is provided for your convenience only.
    // We suggest you write it without modifications.
    
    minifatInfo.supportedSuffixes = supportedFiles;
    player.totalFiles = OpenFile(0xffffU);        
    player.nextStep = 1;
    player.nextFile = 0;
    
    while (1) { // player loop

      if (player.randomOn) {  // Random Play mode 
	player.currentFile = rand() % player.totalFiles;
      } else {                // Normal Play mode
	player.currentFile = player.nextFile;
      }
      
      // "Previous" file selected at the first file
      if (player.currentFile < 0) {
	player.currentFile += player.totalFiles;
      }

      // "Next" file selected at or after the last file
      if (player.currentFile >= player.totalFiles){
	player.currentFile -= player.totalFiles;
      }

      // Try to open file number player.currentFile
      if (OpenFile(player.currentFile) < 0) { 
	// File opened. (OpenFile returns negative number for ok
	// or number of files if not ok)
	player.nextFile = player.currentFile + 1;
	{
	  register s_int16 oldStep = player.nextStep;
	  register s_int16 result = PlayCurrentFile(); // Play File.

	  // If the file format was unsupported, skip to next file
	  if (result == ceFormatNotFound) {
	    player.nextFile = player.currentFile + player.nextStep;
	  }
	  // restore play direction
	  if (result == ceOk && oldStep == -1) {
	    player.nextStep = 1;
	  }
	}
      } else { // Could not OpenFile.
	player.nextFile = 0; // Revert to playing the first file
      }
    } //end of player loop
  } 
}


